/**
 * ScriptDev3 is an extension for mangos providing enhanced features for
 * area triggers, creatures, game objects, instances, items, and spells beyond
 * the default database scripting in mangos.
 *
 * Copyright (C) 2006-2013  ScriptDev2 <http://www.scriptdev2.com/>
 * Copyright (C) 2014-2018  MaNGOS  <https://getmangos.eu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * World of Warcraft, and all World of Warcraft or Warcraft art, images,
 * and lore are copyrighted by Blizzard Entertainment, Inc.
 */

/**
 * ScriptData
 * SDName:      Boss_Magmadar
 * SD%Complete: 75
 * SDComment:   Lava bomb needs still core support
 * SDCategory:  Molten Core
 * EndScriptData
 */

#include "precompiled.h"
#include "molten_core.h"

enum
{
    EMOTE_GENERIC_FRENZY_KILL   = -1000001,

    SPELL_FRENZY                = 19451,
    SPELL_MAGMASPIT             = 19449,                    // This is actually a buff he gives himself
    SPELL_PANIC                 = 19408,
    SPELL_LAVABOMB              = 19411,                    // This calls a dummy server side effect that isn't implemented yet
    SPELL_LAVABOMB_ALT          = 19428
};

struct boss_magmadar : public CreatureScript
{
    boss_magmadar() : CreatureScript("boss_magmadar") {}

    struct boss_magmadarAI : public ScriptedAI
    {
        boss_magmadarAI(Creature* pCreature) : ScriptedAI(pCreature)
        {
            m_pInstance = (ScriptedInstance*)pCreature->GetInstanceData();
        }

        ScriptedInstance* m_pInstance;

        uint32 m_uiFrenzyTimer;
        uint32 m_uiPanicTimer;
        uint32 m_uiLavabombTimer;

        void Reset() override
        {
            m_uiFrenzyTimer = 30000;
            m_uiPanicTimer = 7000;
            m_uiLavabombTimer = 12000;
        }

        void Aggro(Unit* /*pWho*/) override
        {
            DoCastSpellIfCan(m_creature, SPELL_MAGMASPIT, true);

            if (m_pInstance)
            {
                m_pInstance->SetData(TYPE_MAGMADAR, IN_PROGRESS);
            }
        }

        void JustDied(Unit* /*pKiller*/) override
        {
            if (m_pInstance)
            {
                m_pInstance->SetData(TYPE_MAGMADAR, DONE);
            }
        }

        void JustReachedHome() override
        {
            if (m_pInstance)
            {
                m_pInstance->SetData(TYPE_MAGMADAR, NOT_STARTED);
            }
        }

        void UpdateAI(const uint32 uiDiff) override
        {
            if (!m_creature->SelectHostileTarget() || !m_creature->getVictim())
            {
                return;
            }

            // Frenzy_Timer
            if (m_uiFrenzyTimer < uiDiff)
            {
                if (DoCastSpellIfCan(m_creature, SPELL_FRENZY) == CAST_OK)
                {
                    DoScriptText(EMOTE_GENERIC_FRENZY_KILL, m_creature);
                    m_uiFrenzyTimer = 15000;
                }
            }
            else
            {
                m_uiFrenzyTimer -= uiDiff;
            }

            // Panic_Timer
            if (m_uiPanicTimer < uiDiff)
            {
                if (DoCastSpellIfCan(m_creature, SPELL_PANIC) == CAST_OK)
                {
                    m_uiPanicTimer = 30000;
                }
            }
            else
            {
                m_uiPanicTimer -= uiDiff;
            }

            // Lavabomb_Timer
            if (m_uiLavabombTimer < uiDiff)
            {
                if (Unit* pTarget = m_creature->SelectAttackingTarget(ATTACKING_TARGET_RANDOM, 0))
                {
                    if (DoCastSpellIfCan(pTarget, SPELL_LAVABOMB) == CAST_OK)
                    {
                        m_uiLavabombTimer = 12000;
                    }
                }
            }
            else
            {
                m_uiLavabombTimer -= uiDiff;
            }

            DoMeleeAttackIfReady();
        }
    };

    CreatureAI* GetAI(Creature* pCreature) override
    {
        return new boss_magmadarAI(pCreature);
    }
};

void AddSC_boss_magmadar()
{
    Script* s;
    s = new boss_magmadar();
    s->RegisterSelf();

    //pNewScript = new Script;
    //pNewScript->Name = "boss_magmadar";
    //pNewScript->GetAI = &GetAI_boss_magmadar;
    //pNewScript->RegisterSelf();
}
