/**
 * ScriptDev3 is an extension for mangos providing enhanced features for
 * area triggers, creatures, game objects, instances, items, and spells beyond
 * the default database scripting in mangos.
 *
 * Copyright (C) 2006-2013  ScriptDev2 <http://www.scriptdev2.com/>
 * Copyright (C) 2014-2018  MaNGOS  <https://getmangos.eu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * World of Warcraft, and all World of Warcraft or Warcraft art, images,
 * and lore are copyrighted by Blizzard Entertainment, Inc.
 */

/**
 * ScriptData
 * SDName:      Thousand_Needles
 * SD%Complete: 90
 * SDComment:   Quest support: 1950, 4770, 4904, 4966
 * SDCategory:  Thousand Needles
 * EndScriptData
 */

/**
 * ContentData
 * npc_kanati
 * npc_lakota_windsong
 * npc_paoka_swiftmountain
 * npc_plucky_johnson
 * EndContentData
 */

#include "precompiled.h"
#include "escort_ai.h"

/*######
# npc_kanati
######*/

enum
{
    SAY_KAN_START              = -1000410,

    QUEST_PROTECT_KANATI        = 4966,
    NPC_GALAK_ASS               = 10720
};

const float m_afGalakLoc[] = { -4867.387695f, -1357.353760f, -48.226f};

struct npc_kanati : public CreatureScript
{
    npc_kanati() : CreatureScript("npc_kanati") {}

    struct npc_kanatiAI : public npc_escortAI
    {
        npc_kanatiAI(Creature* pCreature) : npc_escortAI(pCreature) {}

        void WaypointReached(uint32 uiPointId) override
        {
            switch (uiPointId)
            {
            case 0:
                DoScriptText(SAY_KAN_START, m_creature);
                DoSpawnGalak();
                break;
            case 1:
                if (Player* pPlayer = GetPlayerForEscort())
                {
                    pPlayer->GroupEventHappens(QUEST_PROTECT_KANATI, m_creature);
                }
                break;
            }
        }

        void DoSpawnGalak()
        {
            for (int i = 0; i < 3; ++i)
                m_creature->SummonCreature(NPC_GALAK_ASS,
                m_afGalakLoc[0], m_afGalakLoc[1], m_afGalakLoc[2], 0.0f,
                TEMPSUMMON_TIMED_OOC_DESPAWN, 25000);
        }

        void JustSummoned(Creature* pSummoned) override
        {
            pSummoned->AI()->AttackStart(m_creature);
        }
    };

    CreatureAI* GetAI(Creature* pCreature) override
    {
        return new npc_kanatiAI(pCreature);
    }

    bool OnQuestAccept(Player* pPlayer, Creature* pCreature, const Quest* pQuest) override
    {
        if (pQuest->GetQuestId() == QUEST_PROTECT_KANATI)
        {
            if (npc_kanatiAI* pEscortAI = dynamic_cast<npc_kanatiAI*>(pCreature->AI()))
            {
                pEscortAI->Start(false, pPlayer, pQuest, true);
            }
        }
        return true;
    }
};

/*######
# npc_lakota_windsong
######*/

enum
{
    SAY_LAKO_START              = -1000365,
    SAY_LAKO_LOOK_OUT           = -1000366,
    SAY_LAKO_HERE_COME          = -1000367,
    SAY_LAKO_MORE               = -1000368,
    SAY_LAKO_END                = -1000369,

    QUEST_FREE_AT_LAST          = 4904,
    NPC_GRIM_BANDIT             = 10758,

    ID_AMBUSH_1                 = 0,
    ID_AMBUSH_2                 = 2,
    ID_AMBUSH_3                 = 4
};

float m_afBanditLoc[6][6] =
{
    { -4905.479492f, -2062.732666f, 84.352f},
    { -4915.201172f, -2073.528320f, 84.733f},
    { -4878.883301f, -1986.947876f, 91.966f},
    { -4877.503906f, -1966.113403f, 91.859f},
    { -4767.985352f, -1873.169189f, 90.192f},
    { -4788.861328f, -1888.007813f, 89.888f}
};

struct npc_lakota_windsong : public CreatureScript
{
    npc_lakota_windsong() : CreatureScript("npc_lakota_windsong") {}

    struct npc_lakota_windsongAI : public npc_escortAI
    {
        npc_lakota_windsongAI(Creature* pCreature) : npc_escortAI(pCreature) { }

        void WaypointReached(uint32 uiPointId) override
        {
            switch (uiPointId)
            {
            case 8:
                DoScriptText(SAY_LAKO_LOOK_OUT, m_creature);
                DoSpawnBandits(ID_AMBUSH_1);
                break;
            case 14:
                DoScriptText(SAY_LAKO_HERE_COME, m_creature);
                DoSpawnBandits(ID_AMBUSH_2);
                break;
            case 21:
                DoScriptText(SAY_LAKO_MORE, m_creature);
                DoSpawnBandits(ID_AMBUSH_3);
                break;
            case 45:
                if (Player* pPlayer = GetPlayerForEscort())
                {
                    pPlayer->GroupEventHappens(QUEST_FREE_AT_LAST, m_creature);
                }
                break;
            }
        }

        void DoSpawnBandits(int uiAmbushId)
        {
            for (int i = 0; i < 2; ++i)
                m_creature->SummonCreature(NPC_GRIM_BANDIT,
                m_afBanditLoc[i + uiAmbushId][0], m_afBanditLoc[i + uiAmbushId][1], m_afBanditLoc[i + uiAmbushId][2], 0.0f,
                TEMPSUMMON_TIMED_OOC_OR_DEAD_DESPAWN, 60000);
        }
    };

    CreatureAI* GetAI(Creature* pCreature) override
    {
        return new npc_lakota_windsongAI(pCreature);
    }

    bool OnQuestAccept(Player* pPlayer, Creature* pCreature, const Quest* pQuest) override
    {
        if (pQuest->GetQuestId() == QUEST_FREE_AT_LAST)
        {
            DoScriptText(SAY_LAKO_START, pCreature, pPlayer);
            pCreature->SetFactionTemporary(FACTION_ESCORT_H_NEUTRAL_ACTIVE, TEMPFACTION_RESTORE_RESPAWN);

            if (npc_lakota_windsongAI* pEscortAI = dynamic_cast<npc_lakota_windsongAI*>(pCreature->AI()))
            {
                pEscortAI->Start(false, pPlayer, pQuest);
            }
            return true;
        }
        return false;
    }
};

/*######
# npc_paoka_swiftmountain
######*/

enum
{
    SAY_START           = -1000362,
    SAY_WYVERN          = -1000363,
    SAY_COMPLETE        = -1000364,

    QUEST_HOMEWARD      = 4770,
    NPC_WYVERN          = 4107
};

float m_afWyvernLoc[3][3] =
{
    { -4990.606f, -906.057f, -5.343f},
    { -4970.241f, -927.378f, -4.951f},
    { -4985.364f, -952.528f, -5.199f}
};

struct npc_paoka_swiftmountain : public CreatureScript
{
    npc_paoka_swiftmountain() : CreatureScript("npc_paoka_swiftmountain") {}

    struct npc_paoka_swiftmountainAI : public npc_escortAI
    {
        npc_paoka_swiftmountainAI(Creature* pCreature) : npc_escortAI(pCreature) { }

        void WaypointReached(uint32 uiPointId) override
        {
            switch (uiPointId)
            {
            case 15:
                DoScriptText(SAY_WYVERN, m_creature);
                DoSpawnWyvern();
                break;
            case 26:
                DoScriptText(SAY_COMPLETE, m_creature);
                break;
            case 27:
                if (Player* pPlayer = GetPlayerForEscort())
                {
                    pPlayer->GroupEventHappens(QUEST_HOMEWARD, m_creature);
                }
                break;
            }
        }

        void DoSpawnWyvern()
        {
            for (int i = 0; i < 3; ++i)
                m_creature->SummonCreature(NPC_WYVERN,
                m_afWyvernLoc[i][0], m_afWyvernLoc[i][1], m_afWyvernLoc[i][2], 0.0f,
                TEMPSUMMON_TIMED_OOC_OR_DEAD_DESPAWN, 60000);
        }
    };

    CreatureAI* GetAI(Creature* pCreature) override
    {
        return new npc_paoka_swiftmountainAI(pCreature);
    }

    bool OnQuestAccept(Player* pPlayer, Creature* pCreature, const Quest* pQuest) override
    {
        if (pQuest->GetQuestId() == QUEST_HOMEWARD)
        {
            DoScriptText(SAY_START, pCreature, pPlayer);
            pCreature->SetFactionTemporary(FACTION_ESCORT_H_NEUTRAL_ACTIVE, TEMPFACTION_RESTORE_RESPAWN);

            if (npc_paoka_swiftmountainAI* pEscortAI = dynamic_cast<npc_paoka_swiftmountainAI*>(pCreature->AI()))
            {
                pEscortAI->Start(false, pPlayer, pQuest);
            }
            return true;
        }
        return false;
    }
};

/*######
# "Plucky" Johnson
######*/

enum
{
    FACTION_FRIENDLY        = 35,
    QUEST_SCOOP             = 1950,
    SPELL_PLUCKY_HUMAN      = 9192,
    SPELL_PLUCKY_CHICKEN    = 9220
};

#define GOSSIP_ITEM_QUEST   "Please tell me the Phrase.."   //TODO localise

struct npc_plucky_johnson : public CreatureScript
{
    npc_plucky_johnson() : CreatureScript("npc_plucky_johnson") {}

    struct npc_plucky_johnsonAI : public ScriptedAI
    {
        npc_plucky_johnsonAI(Creature* pCreature) : ScriptedAI(pCreature)
        {
        }

        uint32 m_uiResetTimer;

        void Reset() override
        {
            m_uiResetTimer = 120000;

            if (m_creature->HasFlag(UNIT_NPC_FLAGS, UNIT_NPC_FLAG_GOSSIP))
            {
                m_creature->RemoveFlag(UNIT_NPC_FLAGS, UNIT_NPC_FLAG_GOSSIP);
            }

            m_creature->CastSpell(m_creature, SPELL_PLUCKY_CHICKEN, false);
        }

        void ReceiveEmote(Player* pPlayer, uint32 uiTextEmote) override
        {
            if (pPlayer->GetQuestStatus(QUEST_SCOOP) == QUEST_STATUS_INCOMPLETE)
            {
                if (uiTextEmote == TEXTEMOTE_BECKON)
                {
                    m_creature->SetFactionTemporary(FACTION_FRIENDLY, TEMPFACTION_RESTORE_RESPAWN | TEMPFACTION_RESTORE_COMBAT_STOP);
                    m_creature->SetFlag(UNIT_NPC_FLAGS, UNIT_NPC_FLAG_GOSSIP);
                    m_creature->CastSpell(m_creature, SPELL_PLUCKY_HUMAN, false);
                }
            }

            if (uiTextEmote == TEXTEMOTE_CHICKEN)
            {
                if (m_creature->HasFlag(UNIT_NPC_FLAGS, UNIT_NPC_FLAG_GOSSIP))
                {
                    return;
                }
                else
                {
                    m_creature->SetFactionTemporary(FACTION_FRIENDLY, TEMPFACTION_RESTORE_RESPAWN | TEMPFACTION_RESTORE_COMBAT_STOP);
                    m_creature->SetFlag(UNIT_NPC_FLAGS, UNIT_NPC_FLAG_GOSSIP);
                    m_creature->CastSpell(m_creature, SPELL_PLUCKY_HUMAN, false);
                    m_creature->HandleEmote(EMOTE_ONESHOT_WAVE);
                }
            }
        }

        void UpdateAI(const uint32 uiDiff) override
        {
            if (m_creature->HasFlag(UNIT_NPC_FLAGS, UNIT_NPC_FLAG_GOSSIP))
            {
                if (m_uiResetTimer < uiDiff)
                {
                    if (!m_creature->getVictim())
                    {
                        EnterEvadeMode();
                    }
                    else
                    {
                        m_creature->RemoveFlag(UNIT_NPC_FLAGS, UNIT_NPC_FLAG_GOSSIP);
                    }

                    return;
                }
                else
                {
                    m_uiResetTimer -= uiDiff;
                }
            }

            if (!m_creature->SelectHostileTarget() || !m_creature->getVictim())
            {
                return;
            }

            DoMeleeAttackIfReady();
        }
    };

    CreatureAI* GetAI(Creature* pCreature) override
    {
        return new npc_plucky_johnsonAI(pCreature);
    }

    bool OnGossipHello(Player* pPlayer, Creature* pCreature) override
    {
        if (pPlayer->GetQuestStatus(QUEST_SCOOP) == QUEST_STATUS_INCOMPLETE)
        {
            pPlayer->ADD_GOSSIP_ITEM(GOSSIP_ICON_CHAT, GOSSIP_ITEM_QUEST, GOSSIP_SENDER_MAIN, GOSSIP_ACTION_INFO_DEF);
        }

        pPlayer->SEND_GOSSIP_MENU(720, pCreature->GetObjectGuid());
        return true;
    }

    bool OnGossipSelect(Player* pPlayer, Creature* pCreature, uint32 /*uiSender*/, uint32 uiAction) override
    {
        pPlayer->PlayerTalkClass->ClearMenus();
        if (uiAction == GOSSIP_ACTION_INFO_DEF)
        {
            pPlayer->SEND_GOSSIP_MENU(738, pCreature->GetObjectGuid());
            pPlayer->AreaExploredOrEventHappens(QUEST_SCOOP);
        }

        return true;
    }
};

void AddSC_thousand_needles()
{
    Script* s;
    s = new npc_kanati();
    s->RegisterSelf();
    s = new npc_lakota_windsong();
    s->RegisterSelf();
    s = new npc_paoka_swiftmountain();
    s->RegisterSelf();
    s = new npc_plucky_johnson();
    s->RegisterSelf();

    //pNewScript = new Script;
    //pNewScript->Name = "npc_kanati";
    //pNewScript->GetAI = &GetAI_npc_kanati;
    //pNewScript->pQuestAcceptNPC = &QuestAccept_npc_kanati;
    //pNewScript->RegisterSelf();

    //pNewScript = new Script;
    //pNewScript->Name = "npc_lakota_windsong";
    //pNewScript->GetAI = &GetAI_npc_lakota_windsong;
    //pNewScript->pQuestAcceptNPC = &QuestAccept_npc_lakota_windsong;
    //pNewScript->RegisterSelf();

    //pNewScript = new Script;
    //pNewScript->Name = "npc_paoka_swiftmountain";
    //pNewScript->GetAI = &GetAI_npc_paoka_swiftmountain;
    //pNewScript->pQuestAcceptNPC = &QuestAccept_npc_paoka_swiftmountain;
    //pNewScript->RegisterSelf();

    //pNewScript = new Script;
    //pNewScript->Name = "npc_plucky_johnson";
    //pNewScript->GetAI = &GetAI_npc_plucky_johnson;
    //pNewScript->pGossipHello = &GossipHello_npc_plucky_johnson;
    //pNewScript->pGossipSelect = &GossipSelect_npc_plucky_johnson;
    //pNewScript->RegisterSelf();
}
